// Copyright (c) 2013 by István Váradi

// This file is part of libxplcommon, a common utility library for
// development related to X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef XPLCOMMON_POSIXTHREAD_H
#define XPLCOMMON_POSIXTHREAD_H
//------------------------------------------------------------------------------

#include "config.h"

#ifdef HAVE_PTHREAD_H

//------------------------------------------------------------------------------

#include <pthread.h>

//------------------------------------------------------------------------------

namespace xplcommon {

//------------------------------------------------------------------------------

/**
 * A pthreads-based thread implementation.
 */
class POSIXThread
{
private:
    /**
     * The thread-runner function.
     */
    static void* threadFn(void* arg);

    /**
     * The posix thread handle.
     */
    pthread_t thread;

public:
    /**
     * Virtual destructor.
     */
    virtual ~POSIXThread();

    /**
     * Start the thread.
     *
     * @return if the thread could be started
     */
    bool start();

    /**
     * Perform the thread's real operation.
     */
    virtual void run() = 0;

    /**
     * Wait for the thread to quit.
     */
    void join();
};

//------------------------------------------------------------------------------

/// Common type for OS-independent code
typedef POSIXThread Thread;

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline POSIXThread::~POSIXThread()
{
}

//------------------------------------------------------------------------------

inline bool POSIXThread::start()
{
    return pthread_create(&thread, 0, &threadFn, this)>=0;
}

//------------------------------------------------------------------------------

inline void POSIXThread::join()
{
    pthread_join(thread, 0);
}

//------------------------------------------------------------------------------

} /* namespace xplcommon */

//------------------------------------------------------------------------------
#endif // HAVE_PTHREAD_H

#endif // XPLCOMMON_POSIXTHREAD_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
